@@ -55,6 +55,7 @@ K_PLUGIN_CLASS_WITH_JSON(OpenVpnUiPlugin, "plasmanetworkmanagement_openvpnui.jso
 #define RPORT_TAG "rport"
 #define SECRET_TAG "secret"
 #define TLS_AUTH_TAG "tls-auth"
+#define TLS_CRYPT_TAG "tls-crypt"
 #define TLS_CLIENT_TAG "tls-client"
 #define TLS_REMOTE_TAG "tls-remote"
 #define TUNMTU_TAG "tun-mtu"
@@ -70,6 +71,8 @@ K_PLUGIN_CLASS_WITH_JSON(OpenVpnUiPlugin, "plasmanetworkmanagement_openvpnui.jso
 #define END_KEY_SECRET_TAG "</secret>"
 #define BEGIN_TLS_AUTH_TAG "<tls-auth>"
 #define END_TLS_AUTH_TAG "</tls-auth>"
+#define BEGIN_TLS_CRYPT_TAG "<tls-crypt>"
+#define END_TLS_CRYPT_TAG "</tls-crypt>"
 
 #define PROC_TYPE_TAG "Proc-Type: 4,ENCRYPTED"
 #define PKCS8_TAG "-----BEGIN ENCRYPTED PRIVATE KEY-----"
@@ -142,13 +145,11 @@ OpenVpnUiPlugin::OpenVpnUiPlugin(QObject *parent, const QVariantList &)
 {
 }
 
-OpenVpnUiPlugin::~OpenVpnUiPlugin()
-{
-}
+OpenVpnUiPlugin::~OpenVpnUiPlugin() = default;
 
 SettingWidget *OpenVpnUiPlugin::widget(const NetworkManager::VpnSetting::Ptr &setting, QWidget *parent)
 {
-    OpenVpnSettingWidget *wid = new OpenVpnSettingWidget(setting, parent);
+    auto wid = new OpenVpnSettingWidget(setting, parent);
     return wid;
 }
 
@@ -159,12 +160,12 @@ SettingWidget *OpenVpnUiPlugin::askUser(const NetworkManager::VpnSetting::Ptr &s
 
 QString OpenVpnUiPlugin::suggestedFileName(const NetworkManager::ConnectionSettings::Ptr &connection) const
 {
-    return connection->id() + "_openvpn.conf";
+    return connection->id() + QStringLiteral("_openvpn.conf");
 }
 
 QString OpenVpnUiPlugin::supportedFileExtensions() const
 {
-    return "*.ovpn *.conf";
+    return QStringLiteral("*.ovpn *.conf");
 }
 
 NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileName)
@@ -212,10 +213,10 @@ NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileNam
         QStringList key_value;
         QString line = in.readLine();
         // Skip comments
-        if (line.indexOf('#') >= 0) {
+        if (line.indexOf(QLatin1Char('#')) >= 0) {
             line.truncate(line.indexOf('#'));
         }
-        if (line.indexOf(';') >= 0) {
+        if (line.indexOf(QLatin1Char(';')) >= 0) {
             line.truncate(line.indexOf(';'));
         }
         if (line.isEmpty()) {
@@ -499,6 +500,27 @@ NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileNam
             }
             continue;
         }
+        if (key_value[0] == TLS_CRYPT_TAG && key_value.count() > 1) {
+            key_value[1] = line.right(line.length() - line.indexOf(QRegExp("\\s"))); // Get whole string after key
+
+            // We will copy inline certificate later when we reach <tls-crypt> tag.
+            if (key_value[1].trimmed() != QLatin1String("[inline]")) {
+                if (copyCertificates) {
+                    const QString absoluteFilePath = tryToCopyToCertificatesDirectory(connectionName, unQuote(key_value[1], fileName));
+                    dataMap.insert(QLatin1String(NM_OPENVPN_KEY_TLS_CRYPT), absoluteFilePath);
+                } else {
+                    dataMap.insert(QLatin1String(NM_OPENVPN_KEY_TLS_CRYPT), unQuote(key_value[1], fileName));
+                }
+            }
+
+            if (key_value.count() > 2) {
+                key_value[2] = key_value[1];
+                if (!key_value[2].isEmpty() && (key_value[2].toLong() == 0 || key_value[2].toLong() == 1)) {
+                    dataMap.insert(QLatin1String(NM_OPENVPN_KEY_TA_DIR), key_value[2]);
+                }
+            }
+            continue;
+        }
         if (key_value[0] == CIPHER_TAG) {
             if (key_value.count() == 2) {
                 dataMap.insert(QLatin1String(NM_OPENVPN_KEY_CIPHER), key_value[1]);
@@ -598,6 +620,16 @@ NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileNam
                 }
             }
             continue;
+        } else if (key_value[0] == BEGIN_TLS_CRYPT_TAG) {
+            const QString tlsAuthAbsolutePath = saveFile(in, QLatin1String(END_TLS_CRYPT_TAG), connectionName, "tls_crypt.key");
+            if (!tlsAuthAbsolutePath.isEmpty()) {
+                dataMap.insert(QLatin1String(NM_OPENVPN_KEY_TLS_CRYPT), tlsAuthAbsolutePath);
+
+                if (key_direction > -1) {
+                    dataMap.insert(QLatin1String(NM_OPENVPN_KEY_TA_DIR), QString().setNum(key_direction));
+                }
+            }
+            continue;
         }
 
         // Import X-NM-Routes if present
@@ -605,12 +637,12 @@ NMVariantMapMap OpenVpnUiPlugin::importConnectionSettings(const QString &fileNam
             QList<NetworkManager::IpRoute> list;
             for (int i = 1; i < key_value.count(); i++) {
                 NetworkManager::IpRoute ipRoute;
-                ipRoute.setIp(QHostAddress(key_value[1].split('/').first()));
-                ipRoute.setPrefixLength(key_value[1].split('/').at(1).toInt());
+                ipRoute.setIp(QHostAddress(key_value[1].split(QLatin1Char('/')).first()));
+                ipRoute.setPrefixLength(key_value[1].split(QLatin1Char('/')).at(1).toInt());
                 list << ipRoute;
             }
             QList<QList<uint>> dbusRoutes;
-            for (const NetworkManager::IpRoute &route : list) {
+            for (const NetworkManager::IpRoute &route : qAsConst(list)) {
                 QList<uint> dbusRoute;
                 dbusRoute << htonl(route.ip().toIPv4Address()) << route.prefixLength() << htonl(route.nextHop().toIPv4Address()) << route.metric();
                 dbusRoutes << dbusRoute;
@@ -695,7 +727,7 @@ QString OpenVpnUiPlugin::saveFile(QTextStream &in, const QString &endTag, const
     QDir().mkpath(certificatesDirectory);
     if (!outFile.open(QFile::WriteOnly | QFile::Text)) {
         KMessageBox::information(nullptr, i18n("Error saving file %1: %2", absoluteFilePath, outFile.errorString()));
-        return QString();
+        return {};
     }
 
     QTextStream out(&outFile);
